-- ===================================================================
-- /orgs.lua
-- User interaction functions for Modular Politics
-- Called by modpol.lua

modpol.interactions = {}


-- DASHBOARDS
-- ==========

function modpol.interactions.login()
   print("Log in as which user?")
   local username = io.read()

   while true do
      local org = modpol.interactions.dashboard(username)
      if org then
         modpol.interactions.org_dashboard(username, org)
      else
         break
      end
   end
end

-- Function: modpol.interactions.dashboard(user)
-- Params: user (string)
-- Q: Should this return a menu of commands relevant to the specific user?
-- Output: Displays a menu of commands to the user
-- TKTK currently just prints all of modpol---needs major improvement
function modpol.interactions.dashboard(user)
   print()
   local org_list = modpol.orgs.list_all()

   print('Select an org')
   for i, org_name in ipairs(org_list) do
      local org = modpol.orgs.get_org(org_name)
      local indicator = ""
      if org:has_pending_actions(user) then
         indicator = "*"
      end
      print('['..i..']'..' '..org_name..indicator)
   end

   local sel = io.read()
   local sel_org = org_list[tonumber(sel)]

   if not sel_org then return end
   
   modpol.interactions.org_dashboard(user, sel_org)
   
end


-- Function: modpol.interactions.org_dashboard
-- Params: user (string), org_name (string)
-- Output: Displays a menu of org-specific commands to the user
function modpol.interactions.org_dashboard(user, org_name)
   print()
   local org = modpol.orgs.get_org(org_name)
   if not org then return nil end
   
   
   local children = {}
   for k,v in ipairs(org.children) do
      local this_child = modpol.orgs.get_org(v)
      table.insert(children, this_child.name)
   end

   local processes = {}
   for k,v in ipairs(org.processes) do
      print(k, v)
      local this_request = org.requests[v.request_id]
      if type(this_request) == "table" then 
         local active = ''
         if org.pending[user] then
            if org.pending[user][v.id] then
               active = '*'
            end
         end
         local req_str = v.id .. " (" .. this_request.type .. " -> " .. table.concat(this_request.params, ", ") .. ")" .. active
         table.insert(processes, req_str)
      end
   end
   -- set up output
   print("Org: " .. org_name)
   print("Members: " .. table.concat(org.members, ", "))
   print("Children: " .. table.concat(children, ", "))
   print("Processes: " .. table.concat(processes, ", "))
   
   print('Interact with which process?')
   local sel = io.read()
   local process = org.processes[tonumber(sel)]
   if not process then return end
   process:interact(user)
end

-- ===================================================================
-- Function: modpol.interactions.message
-- input: user (string), message (string)
-- output: prints message to CLI
function modpol.interactions.message(user, message)
   print(user .. ": " .. message)
end

-- ===================================================================
-- Function: modpol.interactions.text_query
-- input: Query (string)
-- output: User response (string)
function modpol.interactions.text_query(query)
   -- TODO
end

-- ===================================================================
-- Function: modpol.binary_poll_user(user, question)
-- Params: user (string), question (string)
-- Output:
-- presents a yes/no/abstain poll to a user, returns answer
function modpol.interactions.binary_poll_user(user, question)
   local query = "Poll for " .. user .. " (y/n): ".. question
   local answer
   repeat
      print(query)
      answer = io.read()
   until answer == "y" or answer == "n" or answer == "a"
   if answer == "y" then
      return "yes"
   elseif answer == "n" then
      return "no"
   else
      return "abstain"
   end
end
